"""Tests for MCP server functionality."""

import pytest
from unittest.mock import patch, AsyncMock
import mcp.types as types
from domainr_mcp_server.server import handle_list_tools, handle_call_tool


class TestMCPServer:
    """Test cases for MCP server handlers."""

    @pytest.mark.asyncio
    async def test_list_tools(self):
        """Test that all tools are properly listed."""
        tools = await handle_list_tools()
        
        assert len(tools) == 3
        
        tool_names = [tool.name for tool in tools]
        assert "search_domains" in tool_names
        assert "check_domain_status" in tool_names
        assert "register_domain" in tool_names
        
        # Check search_domains tool structure
        search_tool = next(tool for tool in tools if tool.name == "search_domains")
        assert search_tool.description == "Search for domain names based on a query string"
        assert "query" in search_tool.inputSchema["properties"]
        assert "query" in search_tool.inputSchema["required"]
        assert "location" in search_tool.inputSchema["properties"]
        assert "registrar" in search_tool.inputSchema["properties"]
        assert "defaults" in search_tool.inputSchema["properties"] 
        assert "keywords" in search_tool.inputSchema["properties"]

    @pytest.mark.asyncio
    async def test_search_domains_success(self):
        """Test successful domain search through MCP."""
        mock_result = {
            "results": [
                {"domain": "example.com", "zone": "com", "availability": "active"},
                {"domain": "example.org", "zone": "org", "availability": "inactive"}
            ],
            "related": ["sample.com", "demo.com"]
        }

        with patch("domainr_mcp_server.server.domainr_client.search_domains") as mock_search:
            mock_search.return_value = mock_result
            
            result = await handle_call_tool("search_domains", {"query": "example"})
            
            assert len(result) == 1
            assert isinstance(result[0], types.TextContent)
            assert "Domain search results for 'example'" in result[0].text
            assert "example.com (active) - com" in result[0].text
            assert "example.org (inactive) - org" in result[0].text
            assert "Related suggestions: sample.com, demo.com" in result[0].text
            
            mock_search.assert_called_once_with("example", "us", None, None, None)

    @pytest.mark.asyncio
    async def test_search_domains_with_params(self):
        """Test domain search with all parameters."""
        mock_result = {"results": []}

        with patch("domainr_mcp_server.server.domainr_client.search_domains") as mock_search:
            mock_search.return_value = mock_result
            
            args = {
                "query": "test",
                "location": "uk",
                "registrar": "dnsimple.com", 
                "defaults": "com,org",
                "keywords": "tech,startup"
            }
            
            await handle_call_tool("search_domains", args)
            
            mock_search.assert_called_once_with(
                "test", "uk", "dnsimple.com", "com,org", "tech,startup"
            )

    @pytest.mark.asyncio
    async def test_search_domains_no_query(self):
        """Test domain search without required query parameter."""
        result = await handle_call_tool("search_domains", {})
        
        assert len(result) == 1
        assert isinstance(result[0], types.TextContent)
        assert "Error: query parameter is required" in result[0].text

    @pytest.mark.asyncio
    async def test_search_domains_no_results(self):
        """Test domain search with no results."""
        mock_result = {}

        with patch("domainr_mcp_server.server.domainr_client.search_domains") as mock_search:
            mock_search.return_value = mock_result
            
            result = await handle_call_tool("search_domains", {"query": "nonexistent"})
            
            assert len(result) == 1
            assert isinstance(result[0], types.TextContent)
            assert "No results found for query: nonexistent" in result[0].text

    @pytest.mark.asyncio
    async def test_check_domain_status_success(self):
        """Test successful domain status check through MCP."""
        mock_result = {
            "status": [
                {"domain": "example.com", "summary": "active"},
                {"domain": "example.org", "summary": "inactive"}
            ]
        }

        with patch("domainr_mcp_server.server.domainr_client.check_status") as mock_check:
            mock_check.return_value = mock_result
            
            result = await handle_call_tool("check_domain_status", {
                "domains": "example.com,example.org"
            })
            
            assert len(result) == 1
            assert isinstance(result[0], types.TextContent)
            assert "Domain status check:" in result[0].text
            assert "example.com: active" in result[0].text
            assert "example.org: inactive" in result[0].text
            
            mock_check.assert_called_once_with("example.com,example.org")

    @pytest.mark.asyncio
    async def test_check_domain_status_no_domains(self):
        """Test domain status check without required domains parameter."""
        result = await handle_call_tool("check_domain_status", {})
        
        assert len(result) == 1
        assert isinstance(result[0], types.TextContent)
        assert "Error: domains parameter is required" in result[0].text

    @pytest.mark.asyncio
    async def test_check_domain_status_no_results(self):
        """Test domain status check with no results."""
        mock_result = {}

        with patch("domainr_mcp_server.server.domainr_client.check_status") as mock_check:
            mock_check.return_value = mock_result
            
            result = await handle_call_tool("check_domain_status", {
                "domains": "example.com"
            })
            
            assert len(result) == 1
            assert isinstance(result[0], types.TextContent)
            assert "Could not check status for domains: example.com" in result[0].text

    @pytest.mark.asyncio
    async def test_register_domain_success(self):
        """Test successful domain registration URL generation."""
        expected_url = "https://domainr.p.rapidapi.com/v2/register?domain=example.com"

        with patch("domainr_mcp_server.server.domainr_client.get_register_url") as mock_register:
            mock_register.return_value = expected_url
            
            result = await handle_call_tool("register_domain", {"domain": "example.com"})
            
            assert len(result) == 1
            assert isinstance(result[0], types.TextContent)
            assert "Registration URL for 'example.com':" in result[0].text
            assert expected_url in result[0].text
            assert "suitable registrar" in result[0].text
            
            mock_register.assert_called_once_with("example.com", None)

    @pytest.mark.asyncio
    async def test_register_domain_with_registrar(self):
        """Test domain registration URL with specific registrar."""
        expected_url = "https://domainr.p.rapidapi.com/v2/register?domain=example.com&registrar=dnsimple.com"

        with patch("domainr_mcp_server.server.domainr_client.get_register_url") as mock_register:
            mock_register.return_value = expected_url
            
            result = await handle_call_tool("register_domain", {
                "domain": "example.com",
                "registrar": "dnsimple.com"
            })
            
            assert len(result) == 1
            assert isinstance(result[0], types.TextContent)
            assert expected_url in result[0].text
            assert "redirect you to dnsimple.com" in result[0].text
            
            mock_register.assert_called_once_with("example.com", "dnsimple.com")

    @pytest.mark.asyncio
    async def test_register_domain_no_domain(self):
        """Test domain registration without required domain parameter."""
        result = await handle_call_tool("register_domain", {})
        
        assert len(result) == 1
        assert isinstance(result[0], types.TextContent)
        assert "Error: domain parameter is required" in result[0].text

    @pytest.mark.asyncio
    async def test_unknown_tool(self):
        """Test calling unknown tool."""
        result = await handle_call_tool("unknown_tool", {})
        
        assert len(result) == 1
        assert isinstance(result[0], types.TextContent)
        assert "Unknown tool: unknown_tool" in result[0].text

    @pytest.mark.asyncio
    async def test_value_error_handling(self):
        """Test handling of ValueError (missing API key)."""
        with patch("domainr_mcp_server.server.domainr_client.search_domains") as mock_search:
            mock_search.side_effect = ValueError("Domainr API key is required")
            
            result = await handle_call_tool("search_domains", {"query": "test"})
            
            assert len(result) == 1
            assert isinstance(result[0], types.TextContent)
            assert "Configuration error" in result[0].text
            assert "DOMAINR_API_KEY environment variable" in result[0].text

    @pytest.mark.asyncio
    async def test_http_error_handling(self):
        """Test handling of HTTP errors."""
        import httpx
        
        with patch("domainr_mcp_server.server.domainr_client.search_domains") as mock_search:
            mock_response = AsyncMock()
            mock_response.status_code = 404
            mock_response.text = "Not Found"
            
            mock_search.side_effect = httpx.HTTPStatusError(
                "404 Not Found", request=None, response=mock_response
            )
            
            result = await handle_call_tool("search_domains", {"query": "test"})
            
            assert len(result) == 1
            assert isinstance(result[0], types.TextContent)
            assert "API error: 404 - Not Found" in result[0].text

    @pytest.mark.asyncio
    async def test_generic_error_handling(self):
        """Test handling of generic exceptions."""
        with patch("domainr_mcp_server.server.domainr_client.search_domains") as mock_search:
            mock_search.side_effect = Exception("Something went wrong")
            
            result = await handle_call_tool("search_domains", {"query": "test"})
            
            assert len(result) == 1
            assert isinstance(result[0], types.TextContent)
            assert "Error: Something went wrong" in result[0].text


import asyncio
import os
from typing import Any, Dict, List, Optional
import httpx
from mcp.server.models import InitializationOptions
import mcp.types as types
from mcp.server import NotificationOptions, Server
import mcp.server.stdio


class DomainrClient:
    """Client for interacting with the Domainr API."""

    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key or os.getenv("DOMAINR_API_KEY")
        self.base_url = "https://domainr.p.rapidapi.com/v2"
        self.headers = {
            "X-RapidAPI-Key": self.api_key,
            "X-RapidAPI-Host": "domainr.p.rapidapi.com",
        }

    async def search_domains(
        self,
        query: str,
        location: str = "us",
        registrar: Optional[str] = None,
        defaults: Optional[str] = None,
        keywords: Optional[str] = None,
    ) -> Dict[str, Any]:
        """Search for domains based on a query."""
        if not self.api_key:
            raise ValueError("Domainr API key is required")

        params = {"query": query, "location": location}
        if registrar:
            params["registrar"] = registrar
        if defaults:
            params["defaults"] = defaults
        if keywords:
            params["keywords"] = keywords

        async with httpx.AsyncClient() as client:
            response = await client.get(
                f"{self.base_url}/search", headers=self.headers, params=params
            )
            response.raise_for_status()
            return response.json()

    async def check_status(self, domains: str) -> Dict[str, Any]:
        """Check the status of one or more domains."""
        if not self.api_key:
            raise ValueError("Domainr API key is required")

        async with httpx.AsyncClient() as client:
            response = await client.get(
                f"{self.base_url}/status",
                headers=self.headers,
                params={"domain": domains},
            )
            response.raise_for_status()
            return response.json()

    async def get_register_url(
        self, domain: str, registrar: Optional[str] = None
    ) -> str:
        """Get registration URL for a domain."""
        if not self.api_key:
            raise ValueError("Domainr API key is required")

        params = {"domain": domain}
        if registrar:
            params["registrar"] = registrar

        # Build the registration URL
        query_string = "&".join(f"{k}={v}" for k, v in params.items())
        return f"{self.base_url}/register?{query_string}"


# Initialize the MCP server
server = Server("domainr-server")

# Initialize Domainr client
domainr_client = DomainrClient()


@server.list_tools()
async def handle_list_tools() -> List[types.Tool]:
    """List available tools."""
    return [
        types.Tool(
            name="search_domains",
            description="Search for domain names based on a query string",
            inputSchema={
                "type": "object",
                "properties": {
                    "query": {
                        "type": "string",
                        "description": "The search query for domain names",
                    },
                    "location": {
                        "type": "string",
                        "description": "Location code for search context (default: 'us')",
                        "default": "us",
                    },
                    "registrar": {
                        "type": "string",
                        "description": "Filter results by zones supported by this registrar (e.g. 'dnsimple.com')",
                    },
                    "defaults": {
                        "type": "string",
                        "description": "Comma-separated list of default zones to include (e.g. 'club,org')",
                    },
                    "keywords": {
                        "type": "string",
                        "description": "Comma-separated keywords for seeding results (e.g. 'food,kitchen')",
                    },
                },
                "required": ["query"],
            },
        ),
        types.Tool(
            name="check_domain_status",
            description="Check the availability status of one or more domains",
            inputSchema={
                "type": "object",
                "properties": {
                    "domains": {
                        "type": "string",
                        "description": "Comma-separated list of domain names to check",
                    }
                },
                "required": ["domains"],
            },
        ),
        types.Tool(
            name="register_domain",
            description="Get registration URL for a domain name",
            inputSchema={
                "type": "object",
                "properties": {
                    "domain": {
                        "type": "string",
                        "description": "Domain name to register (e.g. 'example.com')",
                    },
                    "registrar": {
                        "type": "string",
                        "description": "Preferred registrar URL (e.g. 'dnsimple.com') - optional",
                    },
                },
                "required": ["domain"],
            },
        ),
    ]


@server.call_tool()
async def handle_call_tool(
    name: str, arguments: Dict[str, Any]
) -> List[types.TextContent]:
    """Handle tool calls."""
    try:
        if name == "search_domains":
            query = arguments.get("query")
            location = arguments.get("location", "us")
            registrar = arguments.get("registrar")
            defaults = arguments.get("defaults")
            keywords = arguments.get("keywords")

            if not query:
                return [
                    types.TextContent(
                        type="text", text="Error: query parameter is required"
                    )
                ]

            result = await domainr_client.search_domains(
                query, location, registrar, defaults, keywords
            )

            # Format the results nicely
            if "results" in result:
                formatted_results = []
                for domain in result["results"]:
                    status = domain.get("availability", "unknown")
                    zone = domain.get("zone", "")
                    formatted_results.append(
                        f"• {domain['domain']} ({status}) - {zone}"
                    )

                response = f"Domain search results for '{query}':\n\n" + "\n".join(
                    formatted_results
                )

                if "related" in result and result["related"]:
                    response += (
                        f"\n\nRelated suggestions: {', '.join(result['related'])}"
                    )
            else:
                response = f"No results found for query: {query}"

            return [types.TextContent(type="text", text=response)]

        elif name == "check_domain_status":
            domains = arguments.get("domains")

            if not domains:
                return [
                    types.TextContent(
                        type="text", text="Error: domains parameter is required"
                    )
                ]

            result = await domainr_client.check_status(domains)

            # Format the status results
            if "status" in result:
                formatted_status = []
                for status_info in result["status"]:
                    domain = status_info.get("domain", "unknown")
                    summary = status_info.get("summary", "unknown")
                    formatted_status.append(f"• {domain}: {summary}")

                response = "Domain status check:\n\n" + "\n".join(formatted_status)
            else:
                response = f"Could not check status for domains: {domains}"

            return [types.TextContent(type="text", text=response)]

        elif name == "register_domain":
            domain = arguments.get("domain")
            registrar = arguments.get("registrar")

            if not domain:
                return [
                    types.TextContent(
                        type="text", text="Error: domain parameter is required"
                    )
                ]

            register_url = await domainr_client.get_register_url(domain, registrar)

            response = f"Registration URL for '{domain}':\n\n{register_url}"
            if registrar:
                response += f"\n\nThis URL will redirect you to {registrar} to complete the registration."
            else:
                response += "\n\nThis URL will redirect you to a suitable registrar to complete the registration."

            return [types.TextContent(type="text", text=response)]

        else:
            return [types.TextContent(type="text", text=f"Unknown tool: {name}")]

    except ValueError as e:
        return [
            types.TextContent(
                type="text",
                text=f"Configuration error: {str(e)}. Please set DOMAINR_API_KEY environment variable.",
            )
        ]
    except httpx.HTTPStatusError as e:
        return [
            types.TextContent(
                type="text",
                text=f"API error: {e.response.status_code} - {e.response.text}",
            )
        ]
    except Exception as e:
        return [types.TextContent(type="text", text=f"Error: {str(e)}")]


async def run_server():
    # Run the server using stdin/stdout streams
    async with mcp.server.stdio.stdio_server() as (read_stream, write_stream):
        await server.run(
            read_stream,
            write_stream,
            InitializationOptions(
                server_name="domainr-server",
                server_version="0.1.0",
                capabilities=server.get_capabilities(
                    notification_options=NotificationOptions(),
                    experimental_capabilities={},
                ),
            ),
        )


def main():
    """Entry point for the MCP server."""
    asyncio.run(run_server())


if __name__ == "__main__":
    main()


"""Tests for DomainrClient."""

import pytest
import httpx
from unittest.mock import patch
from domainr_mcp_server.server import DomainrClient


class TestDomainrClient:
    """Test cases for DomainrClient class."""

    def test_init_with_api_key(self):
        """Test client initialization with API key."""
        client = DomainrClient(api_key="test_key")
        assert client.api_key == "test_key"
        assert client.base_url == "https://domainr.p.rapidapi.com/v2"
        assert client.headers["X-RapidAPI-Key"] == "test_key"
        assert client.headers["X-RapidAPI-Host"] == "domainr.p.rapidapi.com"

    @patch.dict("os.environ", {"DOMAINR_API_KEY": "env_key"})
    def test_init_with_env_var(self):
        """Test client initialization with environment variable."""
        client = DomainrClient()
        assert client.api_key == "env_key"

    def test_init_without_api_key(self):
        """Test client initialization without API key."""
        with patch.dict("os.environ", {}, clear=True):
            client = DomainrClient()
            assert client.api_key is None

    @pytest.mark.asyncio
    async def test_search_domains_success(self, httpx_mock):
        """Test successful domain search."""
        client = DomainrClient(api_key="test_key")

        mock_response = {
            "results": [
                {"domain": "example.com", "zone": "com", "availability": "active"},
                {"domain": "example.org", "zone": "org", "availability": "inactive"},
            ],
            "related": ["sample.com", "demo.com"],
        }

        httpx_mock.add_response(
            url="https://domainr.p.rapidapi.com/v2/search?query=example&location=us",
            json=mock_response,
        )

        result = await client.search_domains("example")
        assert result == mock_response

    @pytest.mark.asyncio
    async def test_search_domains_with_all_params(self, httpx_mock):
        """Test domain search with all optional parameters."""
        client = DomainrClient(api_key="test_key")

        httpx_mock.add_response(
            url="https://domainr.p.rapidapi.com/v2/search?query=test&location=uk&registrar=dnsimple.com&defaults=com%2Corg&keywords=tech%2Cstartup",
            json={"results": []},
        )

        result = await client.search_domains(
            query="test",
            location="uk",
            registrar="dnsimple.com",
            defaults="com,org",
            keywords="tech,startup",
        )

        assert result == {"results": []}

    @pytest.mark.asyncio
    async def test_search_domains_no_api_key(self):
        """Test domain search without API key raises error."""
        client = DomainrClient()

        with pytest.raises(ValueError, match="Domainr API key is required"):
            await client.search_domains("example")

    @pytest.mark.asyncio
    async def test_search_domains_http_error(self, httpx_mock):
        """Test domain search with HTTP error."""
        client = DomainrClient(api_key="test_key")

        httpx_mock.add_response(
            url="https://domainr.p.rapidapi.com/v2/search?query=example&location=us",
            status_code=404,
            text="Not Found",
        )

        with pytest.raises(httpx.HTTPStatusError):
            await client.search_domains("example")

    @pytest.mark.asyncio
    async def test_check_status_success(self, httpx_mock):
        """Test successful domain status check."""
        client = DomainrClient(api_key="test_key")

        mock_response = {
            "status": [
                {
                    "domain": "example.com",
                    "zone": "com",
                    "status": "active",
                    "summary": "active",
                }
            ]
        }

        httpx_mock.add_response(
            url="https://domainr.p.rapidapi.com/v2/status?domain=example.com%2Cexample.org",
            json=mock_response,
        )

        result = await client.check_status("example.com,example.org")
        assert result == mock_response

    @pytest.mark.asyncio
    async def test_check_status_no_api_key(self):
        """Test status check without API key raises error."""
        client = DomainrClient()

        with pytest.raises(ValueError, match="Domainr API key is required"):
            await client.check_status("example.com")

    @pytest.mark.asyncio
    async def test_check_status_http_error(self, httpx_mock):
        """Test status check with HTTP error."""
        client = DomainrClient(api_key="test_key")

        httpx_mock.add_response(
            url="https://domainr.p.rapidapi.com/v2/status?domain=example.com",
            status_code=500,
            text="Internal Server Error",
        )

        with pytest.raises(httpx.HTTPStatusError):
            await client.check_status("example.com")

    @pytest.mark.asyncio
    async def test_get_register_url_basic(self):
        """Test basic registration URL generation."""
        client = DomainrClient(api_key="test_key")

        url = await client.get_register_url("example.com")

        assert url == "https://domainr.p.rapidapi.com/v2/register?domain=example.com"

    @pytest.mark.asyncio
    async def test_get_register_url_with_registrar(self):
        """Test registration URL generation with registrar."""
        client = DomainrClient(api_key="test_key")

        url = await client.get_register_url("example.com", "dnsimple.com")

        expected = "https://domainr.p.rapidapi.com/v2/register?domain=example.com&registrar=dnsimple.com"
        assert url == expected

    @pytest.mark.asyncio
    async def test_get_register_url_no_api_key(self):
        """Test registration URL generation without API key raises error."""
        client = DomainrClient()

        with pytest.raises(ValueError, match="Domainr API key is required"):
            await client.get_register_url("example.com")
